import Helper from '@ember/component/helper';
import { isNone } from '@ember/utils';
import { htmlSafe } from '@ember/string';
import styles from '@choiceform/ui-foundation/styles/icon';

export function icon([
  name = null
], {
  image = null,
  type = null,
  size = null,
  viewBox = null,
  radius = 0,
  color = null,
  fill = null,
}) {
  if (isNone(size)) {
    switch (type) {
      case 'symbol': size = 36; break;
      case 'logo': size = 48; break;
      default: size = 16;
    }
  } else if (isNone(viewBox)) {
    switch (type) {
      case 'symbol': viewBox = size; break;
      case 'logo': viewBox = size; break;
      default: viewBox = size;
    }
  }

  if (isNone(viewBox)) {
    switch(type) {
      case 'symbol': viewBox = 36; break;
      case 'logo': viewBox = 48; break;
      default: viewBox = 16;
    }
  }

  if (name === null && image) {
    return htmlSafe(renderImageIcon(image, size, radius));
  } else {
    return htmlSafe(renderSVGIcon(name, type, size, viewBox, color, fill));
  }
}

export function renderImageIcon(image, size, radius, className = 'ui-profile') {
  className = `ui-profile ${styles.image}`;
  let style = `border-radius: ${radius}px; background-image: url(${image}); width: ${size}px; height: ${size}px;`;
  return `<div class="${className}" style="${style}"></div>`;
}

export function renderSVGIcon(name, type, size, viewBox, color, fill, className = 'ui-icon') {
  // symbol type
  if (type === 'symbol') {
    className += ` ${styles[type]}`;
    if (color) className += ` ${styles[color]}`;
    color = null;
  }

  // logo type
  if (type === 'logo') {
    className += ` ${styles[type]}`;
  }

  color = color ? `fill="${color}" ` : '';
  return `<svg class="${className}" ` +
    `xmlns="http://www.w3.org/2000/svg" ` +
    `xmlns:xlink="http://www.w3.org/1999/xlink" ` +
    `x="${size}" y="${size}" width="${size}" height="${size}" ` +
    `viewBox="0 0 ${viewBox} ${viewBox}" preserveRatioAspect="xMidYMid">` +
    `${renderSpecialShape(type, fill, size)}` +
    `<use ${color}xlink:href="#${name}"></use></svg>`;
}

export function renderSpecialShape(type, fill, size, rx) {
  switch (type) {
    case 'logo': rx = 4; break;
    case 'symbol': rx = 12; break;
    default: return '';
  }

  fill = fill === null ? '' : `fill="${fill}" `;
  return `<rect ${fill}x="0" y="0" width="${size}" height="${size}" rx="${rx}"></rect>`;
}

export default Helper.helper(icon);
